#include <LiquidCrystal.h>
#include <Wire.h>
#include "LSM303.h"
LSM303 akcelerometr;
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);    
float acc_x, acc_y, acc_z, mag_x, mag_y, mag_z, kat; //zmienne w których przechowywane będą wyskalowane odczyty czujnika
float ax_offset = 0, ay_offset = 0, az_offset = 0; //zmienne kalibrujące odczyty akcelerometru
void wyswietlPrzyspieszenie(void); //deklaracja funkcji odczytującej i wyswietlajacej wskazania akcelerometru
void wyswietlKompas(void); //deklaracja funkcji odczytującej i wyświetlającej wskazania kompasu
void wyswietlMagnetometr (void); //deklaracja funkcji odczytującej i wyswietlajacej wskazania magnetometru
void (*wskaznikFunkcji) (void); //wskaźnik na funkcję która ma zostać wykonana w danej iteracji pętli
void (*tablicaFunkcji[3])(void) = {wyswietlPrzyspieszenie, wyswietlMagnetometr, wyswietlKompas}; //tablica wskaźników do funckji odczytujacych i wyswietlajacych dane
unsigned short int f=0; // numer aktualnie używanej funkcji
int stat; //odczyt z linii A0, podłączonej do klawiatury

void setup() {
  Wire.begin(); //deklaracja połączenia I2C
  lcd.begin(16,2); //inicjalizacja ekranu       
  lcd.setCursor(0,0);
  akcelerometr.init(); //inicjalizacja czujnika
  akcelerometr.enableDefault();
  kalibracja(); //wywołanie procedury kalibracyjnej
  wskaznikFunkcji = tablicaFunkcji[f];
}

void loop() {
  klawiatura();
  wskaznikFunkcji();
}

void klawiatura (void){ //Odczyt klawiatury i wybór funkcji
  stat = analogRead(A0);
   if (stat > 1000) return; 
   if (stat < 250 && stat > 50) { // naciśnięcie przycisku UP
    wskaznikFunkcji = tablicaFunkcji[(++f)%=3];
    delay(100);
    return;
   }
  if (stat < 450 && stat > 250 )  { // naciśnięcie przycisku DOWN
    wskaznikFunkcji = tablicaFunkcji[f=(f+2)%3];
    delay(100);
    return;
    }
  if (stat < 850 && stat > 650){  //naciśnięcie przycisku SELECT
    kalibracja();
    return; 
  }
}
void kalibracja(void){ //kalibracja akcelerometru
  const int liczbaProbek = 1024;
  acc_x=0;
  acc_y=0;
  acc_z=0;
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.println("Ustaw czujnik w ");
  lcd.setCursor(0,1);
  lcd.println("pozycji plaskiej");
  delay(5000);
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.println("Kalibracja       ");
  for (int i=0; i<=liczbaProbek; ++i) {
   akcelerometr.read();
    acc_x+=(2.0/32768.0)*akcelerometr.a.x;
    acc_y+=(2.0/32768.0)*akcelerometr.a.y;
    acc_z+=(2.0/32768.0)*akcelerometr.a.z;
  }
  ax_offset = acc_x / liczbaProbek;
  ay_offset = acc_y / liczbaProbek;
  az_offset = (acc_z / liczbaProbek)-1;
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.println("Kalibracja      ");
  lcd.setCursor(0,1);
  lcd.println("Zakonczona      ");
  delay(1500); 
  return;
  }
  
void wyswietlPrzyspieszenie(void) {//definicja funkcji odczytującej i wyświetlającej wskazania akcelerometru
  akcelerometr.readAcc();
  acc_x=((2.0/32768.0)*akcelerometr.a.x)-ax_offset;
  acc_y=((2.0/32768.0)*akcelerometr.a.y)-ay_offset;
  acc_z=((2.0/32768.0)*akcelerometr.a.z)-az_offset;
   lcd.clear();
   lcd.setCursor(0,0);
   lcd.print(acc_x);
   lcd.print(" G");
   lcd.setCursor(8,0);
   lcd.print(acc_y);
   lcd.print(" G");
   lcd.setCursor(0,1);
   lcd.print(acc_z);
   lcd.print(" G");
   lcd.setCursor(10,1);
   lcd.print("ACC");
  delay(100);
}

void wyswietlMagnetometr(void) { //definicja funkcji odczytującej i wyświatlającej wskazania magnetometru
  akcelerometr.readMag(); 
  mag_x=(4.0/32768.0)*akcelerometr.m.x;
  mag_y=(4.0/32768.0)*akcelerometr.m.y;
  mag_z=(4.0/32768.0)*akcelerometr.m.z;
   lcd.clear();
   lcd.setCursor(0,0);
   lcd.print(mag_x);
   lcd.setCursor(8,0);
   lcd.print(mag_y);
   lcd.setCursor(0,1);
   lcd.print(mag_z);
   lcd.setCursor(10,1);
   lcd.print("MAG");
  delay(100);
  }

void wyswietlKompas(void){ //definicja funkcji odczytującej i wyświatlającej wskazania kompasu
    akcelerometr.read();
    acc_x = akcelerometr.heading();
     lcd.clear();
     lcd.setCursor(0,0);
     lcd.print("WYCHYLENIE");
     lcd.setCursor(0,1);
     lcd.print(acc_x);
     lcd.print(" st.");
     delay(100);
  }
  

